-- Puffin: A Tojaso FowlPlay Project
-- Adapter component for configuration data

local Adapter_Configuration = {}
local EF = Puffin.ErrorFormat
local DB
local defaultWhite = { r = 1, g = 1, b = 1, a = 1 }
local defaultBlack = { r = 0, g = 0, b = 0, a = 1 }
local defaultGray = { r = 0.25, g = 0.25, b = 0.25, a = 1 }
local defaultLight = { r = 0.75, g = 0.75, b = 0.75, a = 1 }
local defaultYellow = { r = 1, g = 1, b = 0, a = 1 }
local defaultTransparentBlack = { r = 0, g = 0, b = 0, a = 0.5 }
local defaultTransparentGray = { r = 0.25, g = 0.25, b = 0.25, a = 0.5 }
local defaultTransparentDarkGray = { r = 0.1, g = 0.1, b = 0.1, a = 0.5 }
local defaultTransparent = { r = 0, g = 0, b = 0, a = 0 } 
local mana_color = { r = 0.447, g = 0.62353, b = 0.8118, a = 1 }
local mouseover_color = { r = 1, g = 1, b = 0, a = 0.5 }

-- Defaults applied to all instances of each widget class
local defaults = {
	Widget_Text = {
		font = { "#LSM", "font", "Arial Narrow" }, -- font to use for the text (default: ChatFontNormal) [c]
		fsize = 10, -- font size for the text (default: 10) [c]
		shadow = true, -- enable a shadow behind the text (default: false) [c]
		outline = false, -- enable font "OUTLINE" flag (default: false) [c]
		thick = false, -- enable font "THICKOUTLINE" flag (default: false) [c]
		mono = false, -- enable font "MONOCHROME" flag (default: false) [c]
		wrap = false, -- enable wrap of text to multiple lines (default: false) [c]
		justifyH = "CENTER", -- horizontal text alignment (default: "CENTER") [c]
		justifyV = "MIDDLE", -- vertical text alignment (default: "MIDDLE") [c]
	},
	Widget_Icon = {
	},
	Widget_Rectangle = {
		texture = "Interface\\AddOns\\Puffin\\Images\\TukuiBar",
		borderTexture = "Interface\\AddOns\\Puffin\\Images\\BorderSquare2",
		borderWidth = 4, -- width of border (default: 8) [d]
		borderOffset = 1.25, -- offset of border from texture (default: 2) [d]
	},
	Widget_Panel = {
		panelTexture = { "#LSM", "background", "Solid" }, -- if defined then fill the base with colored texture [c]
		panelInset = 1, -- inset of base texture from border (default: 0) [c]
		panelColor = defaultGray,
	},
	Widget_Bar = {
		fgTexture = "Interface\\AddOns\\Puffin\\Images\\TukuiBar", -- { "#LSM", "statusbar", "Tooltip" },
		bgTexture = "Interface\\AddOns\\Puffin\\Images\\TukuiBar", -- { "#LSM", "statusbar", "Tooltip" }, [c]
		borderTexture = "Interface\\AddOns\\Puffin\\Images\\BorderSquare2",
		borderWidth = 4, -- width of border (default: 8) [c]
		borderOffset = 1.25, -- offset of border from texture (default: 2) [c]
		direction = false, -- which way foreground bar moves as value decreases, false=right-to-left, true=left-to-right (default: false) [c]
		sparkAlpha = 0.5, -- opacity of spark displayed at moving end of the foreground bar (default: 0) [c]
	},
}

local misc_templates = {
	panel = {
		priority = 1, class = "Widget_Panel",
		relativeWidth = 1, relativeHeight = 1, depth = 0,
		attributes = { panelColor = defaultGray, panelAlpha = 0.4, },
	},
	left_stripe = {
		priority = 2, class = "Widget_Panel", parent = "panel", anchor = "panel", anchorPoint = "BOTTOMRIGHT", relativePoint = "BOTTOMLEFT",
		offsetY = 8, width = 470, height = 4, depth = 0,
		attributes = { panelColor = defaultGray, panelAlpha = 0.4, },
	},
	right_stripe = {
		priority = 2, class = "Widget_Panel", parent = "panel", anchor = "panel", anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMRIGHT",
		offsetY = 8, width = 470, height = 4, depth = 0,
		attributes = { panelColor = defaultGray, panelAlpha = 0.4, },
	},
}

local standard_templates = {
	base = {
		priority = 1, class = "Widget_Panel",
		relativeWidth = 1, width = 2, relativeHeight = 1, height = 2, depth = 0,
		attributes = {
			borderOffsetX = -1, borderOffsetY = -1, panelInset = 1, panelAlpha = 0.4, panelColor = defaultGray,
			borderTexture = "Interface\\AddOns\\Puffin\\Images\\BorderSquare2"
		},
		providers = {
			borderColor = { class = "unit_color", name = "threat", param = defaultTransparent },
		},
	},
	healthBar = {
		priority = 2, class = "Widget_Bar", parent = "base", anchor = "base", anchorPoint = "TOPRIGHT", relativePoint = "TOPRIGHT",
		offsetX = -4, offsetY = -4, relativeWidth = 1, width = -4, relativeHeight = 0.55, height = -2, depth = 2,
		attributes = { bgAlpha = 1, sparkAlpha = 0, },
		providers = {
			fgColor = { class = "unit_color", name = "health" },
			bgColor = { class = "unit_color", name = "health_background" },
			value = { class = "unit_number", name = "health_fraction" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	powerBar = {
		priority = 2, class = "Widget_Bar", parent = "base", anchor = "base", anchorPoint = "BOTTOMRIGHT", relativePoint = "BOTTOMRIGHT", 
		offsetX = -4, offsetY = 2, relativeWidth = 1, width = -4, relativeHeight = 0.35, depth = 2,
		attributes = { bgAlpha = 1, sparkAlpha = 0, },
		providers = {
			fgColor = { class = "unit_color", name = "power" },
			bgColor = { class = "unit_color", name = "power_background" },
			value = { class = "unit_number", name = "power_fraction" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	incomingBar = {
		priority = 3, class = "Widget_Rectangle", parent = "base", anchor = "healthBar", anchorRegion = "fgTexture", anchorPoint = "LEFT", relativePoint = "RIGHT",
		relativeWidth = 1, relativeHeight = 0.25, depth = 4, alpha = 0.5,
		attributes = { minimumWidth = 4, valueHeight = 1, },
		providers = {
			color = { class = "unit_color", name = "health" },
			valueWidth = { class = "unit_number", name = "incoming_fraction" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	nameText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "healthBar", anchorPoint = "LEFT", relativePoint = "LEFT",
		offsetX = 3, relativeWidth = 0.6, relativeHeight = 1, depth = 6,
		attributes = { fsize = 10, justifyH = "LEFT", justifyV = "MIDDLE",  },
		providers = {
			text = { class = "unit_text", name = "name" },
			color = { class = "unit_color", name = "class_reaction" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	classText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "powerBar", anchorPoint = "LEFT", relativePoint = "LEFT",
		offsetX = 3, relativeWidth = 0.6, relativeHeight = 1, depth = 6,
		attributes = { fsize = 7, justifyH = "LEFT", justifyV = "MIDDLE", color = defaultWhite  },
		providers = {
			text = { class = "unit_text", name = "level_class" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	healthText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "healthBar", anchorPoint = "RIGHT", relativePoint = "RIGHT",
		offsetX = -3, relativeWidth = 0.4, relativeHeight = 1, depth = 6,
		attributes = { fsize = 8, justifyH = "RIGHT", justifyV = "MIDDLE", color = defaultWhite, },
		providers = {
			text = { class = "unit_text", name = "health_percent" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	powerText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "powerBar", anchorPoint = "RIGHT", relativePoint = "RIGHT",
		offsetX = -3, relativeWidth = 0.4, relativeHeight = 1, depth = 6,
		attributes = { justifyH = "RIGHT", justifyV = "MIDDLE", color = defaultWhite, },
		providers = {
			text = { class = "unit_text", name = "power_status" },
			fsize = { class = "unit_number", name = "not_mana_offset", param = 8 },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	combatIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "BOTTOM", relativePoint = "BOTTOM",
		width = 12, height = 12, depth = 8, offsetY = -1,
		attributes = { texture = [[Interface\CharacterFrame\UI-StateIcon]], texcoord = { .5, 1, 0, .49 }, endAnimation = "fade" },
		providers = {
			condition = { class = "unit_condition", name = "in_combat" },
		},
	},
	pvpIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOPRIGHT",
		width = 12, height = 12, depth = 8, offsetX = -2, offsetY = -2,
		providers = {
			condition = { class = "unit_condition", name = "is_pvp" },
			texture = { class = "unit_icon", name = "pvp" },
		},
	},
	pvpText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "pvpIcon", anchorPoint = "RIGHT", relativePoint = "LEFT",
		relativeWidth = 0.25, height = 8, depth = 8, offsetY = 4,
		attributes = { fsize = 7, justifyH = "RIGHT" },
		providers = {
			text = { class = "unit_text", name = "pvp_timer" },
			color = { class = "color_palette", name = "red" },
		},
	},
	comboText = {
		priority = 2, class = "Widget_Text", parent = "base", anchor = "base", anchorPoint = "BOTTOM", relativePoint = "TOP",
		relativeWidth = 0.5, height = 10, depth = 6,
		attributes = { fsize = 10, justifyH = "CENTER", justifyV = "BOTTOM", },
		providers = {
			text = { class = "unit_text", name = "combo_points" },
			condition = { class = "unit_condition", name = "combo_points" },
		},
	},
	questIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOP",
		width = 12, height = 12, depth = 8, offsetY = -2,
		providers = {
			condition = { class = "unit_condition", name = "quest_boss" },
			texture = { class = "unit_icon", name = "quest_icon" },
		},
	},
	raidIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOP", 
		width = 12, height = 12, scale = 0.75, depth = 8, offsetY = -2,
		providers = {
			condition = { class = "unit_condition", name = "raid_icon" },
			texture = { class = "unit_icon", name = "raid_icon" },
		},
	},
	leaderIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "BOTTOMLEFT", 
		width = 12, height = 12, scale = 0.5, depth = 8, offsetX = 1, offsetY = -1,
		providers = {
			condition = { class = "unit_condition", name = "is_leader" },
			texture = { class = "unit_icon", name = "leader_icon" },
		},
	},
	looterIcon = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "leaderIcon", anchorRegion = "texture", anchorPoint = "TOPLEFT", relativePoint = "TOPRIGHT",
		width = 12, height = 12, scale = 0.5, depth = 8, offsetX = 2,
		providers = {
			condition = { class = "unit_condition", name = "is_master_looter" },
			texture = { class = "unit_icon", name = "master_looter_icon" },
		},
	},
	roleIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "BOTTOMRIGHT",
		width = 12, height = 12, scale = 0.75, alpha = 0.8, offsetX = -2, offsetY = -1, depth = 8,
		providers = {
			condition = { class = "unit_condition", name = "assigned_role" },
			texture = { class = "unit_icon", name = "role_icon" },
		},
	},
	restIcon = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOPLEFT",
		width = 12, height = 12, depth = 8, offsetX = 2, offsetY = -2,
		attributes = { endAnimation = "fade" },
		providers = {
			condition = { class = "unit_condition", name = "is_resting" },
			texture = { class = "unit_icon", name = "rest_icon" },
		},
	},
}

local raid_templates = {
	base = {
		priority = 1, class = "Widget_Panel",
		relativeWidth = 1, width = 2, relativeHeight = 1, height = 2, depth = 0,
		attributes = {
			borderOffsetX = -1, borderOffsetY = -1, panelInset = 1, panelAlpha = 0.4, panelColor = defaultGray,
			borderTexture = "Interface\\AddOns\\Puffin\\Images\\BorderSquare2",
		},
		providers = {
			borderColor = { class = "unit_color", name = "threat", param = defaultTransparent },
		},
	},
	healthBar = {
		priority = 2, class = "Widget_Bar", parent = "base", anchor = "base", anchorPoint = "TOP", relativePoint = "TOP",
		offsetX = -1, offsetY = -4, width = -3, relativeWidth = 1, height = 14, depth = 2,
		attributes = { bgAlpha = 1, sparkAlpha = 0, },
		providers = {
			fgColor = { class = "unit_color", name = "health" },
			bgColor = { class = "unit_color", name = "health_background" },
			value = { class = "unit_number", name = "health_fraction" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	powerBar = {
		priority = 2, class = "Widget_Bar", parent = "base", anchor = "base", anchorPoint = "BOTTOM", relativePoint = "BOTTOM", 
		offsetX = -1, offsetY = 2, width = -3, relativeWidth = 1, height = 4, depth = 2,
		attributes = { bgAlpha = 1, sparkAlpha = 0, },
		providers = {
			fgColor = { class = "unit_color", name = "power" },
			bgColor = { class = "unit_color", name = "power_background" },
			value = { class = "unit_number", name = "power_fraction" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	incomingBar = {
		priority = 3, class = "Widget_Rectangle", parent = "base", anchor = "healthBar", anchorRegion = "fgTexture", anchorPoint = "LEFT", relativePoint = "RIGHT",
		relativeWidth = 1, relativeHeight = 0.4, depth = 4, alpha = 0.5, offsetX = 0.5,
		attributes = { minimumWidth = 2, valueHeight = 1, },
		providers = {
			color = { class = "unit_color", name = "health" },
			valueWidth = { class = "unit_number", name = "incoming_fraction" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	nameText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "healthBar", anchorPoint = "LEFT", relativePoint = "LEFT",
		offsetX = 2, relativeWidth = 1, relativeHeight = 1, depth = 6,
		attributes = { fsize = 9, justifyH = "LEFT", justifyV = "MIDDLE", },
		providers = {
			text = { class = "unit_text", name = "name" },
			color = { class = "unit_color", name = "class_reaction" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	statusText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "powerBar", anchorPoint = "CENTER", relativePoint = "CENTER",
		relativeWidth = 1, relativeHeight = 0.5, depth = 6,
		attributes = { fsize = 7, justifyH = "CENTER", justifyV = "MIDDLE", },
		providers = {
			text = { class = "unit_text", name = "status" },
			color = { class = "unit_color", name = "status" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	readyIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "CENTER",
		width = 16, height = 16, depth = 8,
		attributes = { endAnimation = "delay" },
		providers = {
			condition = { class = "unit_condition", name = "ready_check_active" },
			texture = { class = "unit_icon", name = "ready_icon" },
		},
	},
	arrowIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "CENTER",
		width = 24, height = 24, depth = 8,
		attributes = { endAnimation = "fade" },
		providers = {
			condition = { class = "unit_condition", name = "show_direction" },
			texture = { class = "unit_icon", name = "arrow_icon" },
			rotation = { class = "unit_number", name = "direction" },
			color = { class = "color_palette", name = "yellow" },
		},
	},
	pvpIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOPRIGHT",
		width = 12, height = 12, scale = 0.75, depth = 8, offsetX = -2, offsetY = -2,
		providers = {
			condition = { class = "unit_condition", name = "is_pvp" },
			texture = { class = "unit_icon", name = "pvp" },
		},
	},
	raidIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOP",
		width = 12, height = 12, scale = 0.65, depth = 8, offsetY = -2,
		providers = {
			condition = { class = "unit_condition", name = "raid_icon" },
			texture = { class = "unit_icon", name = "raid_icon" },
		},
	},
	icon1 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "TOPLEFT", relativePoint = "TOPLEFT",
		width = 10, height = 10, depth = 8, offsetY = -2, scale = 0.8,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Indicator", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition1" },
			color = { class = "unit_color", name = "class_color1" },
		},
	},
	text1 = {
		priority = 4, class = "Widget_Text", parent = "base", anchor = "icon1", anchorPoint = "CENTER", relativePoint = "CENTER",
		width = 10, height = 10, depth = 9, offsetX = 1.25, offsetY = 0.25,
		attributes = { fsize = 8, justifyH = "CENTER", justifyV = "MIDDLE", shadow = false, outline = true, color = defaultWhite, textFormat = "time", timeMax = 9.9 },
		providers = {
			condition = { class = "unit_condition", name = "class_condition1" },
			timeExpire = { class = "unit_number", name = "class_time1" },
		},
	},
	icon2 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT",
		width = 10, height = 10, depth = 8, scale = 0.8,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Indicator", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition2" },
			color = { class = "unit_color", name = "class_color2" },
		},
	},
	text2 = {
		priority = 4, class = "Widget_Text", parent = "base", anchor = "icon2", anchorPoint = "CENTER", relativePoint = "CENTER",
		width = 10, height = 10, depth = 9, offsetX = 1.25, offsetY = 0.25,
		attributes = { fsize = 8, justifyH = "CENTER", justifyV = "MIDDLE", shadow = false, outline = true, color = defaultWhite, textFormat = "time", timeMax = 9.9 },
		providers = {
			condition = { class = "unit_condition", name = "class_condition2" },
			timeExpire = { class = "unit_number", name = "class_time2" },
		},
	},
	icon3 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "TOPRIGHT", relativePoint = "TOPRIGHT",
		width = 10, height = 10, depth = 8, offsetX = -2, offsetY = -2, scale = 0.8,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Indicator", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition3" },
			color = { class = "unit_color", name = "class_color3" },
		},
	},
	text3 = {
		priority = 4, class = "Widget_Text", parent = "base", anchor = "icon3", anchorPoint = "CENTER", relativePoint = "CENTER",
		width = 10, height = 10, depth = 9, offsetX = 1.25, offsetY = 0.25,
		attributes = { fsize = 8, justifyH = "CENTER", justifyV = "MIDDLE", shadow = false, outline = true, color = defaultWhite, textFormat = "time", timeMax = 9.9 },
		providers = {
			condition = { class = "unit_condition", name = "class_condition3" },
			timeExpire = { class = "unit_number", name = "class_time2" },
		},
	},
	icon4 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "BOTTOMRIGHT", relativePoint = "BOTTOMRIGHT",
		width = 10, height = 10, depth = 8, offsetX = -2, scale = 0.8,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Indicator", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition4" },
			color = { class = "unit_color", name = "class_color4" },
		},
	},
	text4 = {
		priority = 4, class = "Widget_Text", parent = "base", anchor = "icon4", anchorPoint = "CENTER", relativePoint = "CENTER",
		width = 10, height = 10, depth = 9, offsetX = 1.25, offsetY = 0.25,
		attributes = { fsize = 8, justifyH = "CENTER", justifyV = "MIDDLE", shadow = false, outline = true, color = defaultWhite, textFormat = "time", timeMax = 9.9 },
		providers = {
			condition = { class = "unit_condition", name = "class_condition4" },
			timeExpire = { class = "unit_number", name = "class_time4" },
		},
	},
	icon5 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "BOTTOM", relativePoint = "BOTTOM",
		width = 10, height = 10, depth = 8, scale = 0.8,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Indicator", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition5" },
			color = { class = "unit_color", name = "class_color5" },
		},
	},
	icon6 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "TOPLEFT", relativePoint = "TOPLEFT",
		width = 8, height = 8, depth = 8, offsetX = 12, scale = 0.75,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Circle", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition6" },
			color = { class = "unit_color", name = "class_color6" },
		},
	},
	icon7 = {
		priority = 3, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "TOPRIGHT", relativePoint = "TOPRIGHT",
		width = 8, height = 8, depth = 8, offsetX = -12, scale = 0.75,
		attributes = { texture = "Interface\\Addons\\Puffin\\Images\\Circle", },
		-- attributes = { texture = "Interface\\Icons\\Spell_Nature_Regeneration", },
		providers = {
			condition = { class = "unit_condition", name = "class_condition7" },
			color = { class = "unit_color", name = "class_color7" },
		},
	},
}

local raid_target_templates = {
	base = {
		priority = 1, class = "Widget_Panel",
		relativeWidth = 1, width = 2, relativeHeight = 1, depth = 0,
		attributes = {
			borderOffsetX = -1, borderOffsetY = -1, panelInset = 1, panelAlpha = 0.4, panelColor = defaultGray,
			borderTexture = "Interface\\AddOns\\Puffin\\Images\\BorderSquare2", borderColor = defaultTransparent
		},
	},
	healthBar = {
		priority = 2, class = "Widget_Bar", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "CENTER",
		offsetX = -1, offsetY = 0, width = -3, relativeWidth = 1, height = 9, depth = 2,
		attributes = { bgAlpha = 1, sparkAlpha = 0, },
		providers = {
			fgColor = { class = "unit_color", name = "health" },
			bgColor = { class = "unit_color", name = "health_background" },
			value = { class = "unit_number", name = "health_fraction" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	nameText = {
		priority = 3, class = "Widget_Text", parent = "base", anchor = "healthBar", anchorPoint = "LEFT", relativePoint = "LEFT",
		offsetX = 2, width = -4, relativeWidth = 1, height = 10, depth = 6,
		attributes = { fsize = 7, justifyH = "LEFT", justifyV = "MIDDLE", color = defaultWhite  },
		providers = {
			text = { class = "unit_text", name = "name" },
			color = { class = "unit_color", name = "class_reaction" },
			alpha = { class = "unit_number", name = "range_alpha" },
		},
	},
	raidIcon = {
		priority = 2, class = "Widget_Icon", parent = "base", anchor = "base", anchorPoint = "CENTER", relativePoint = "TOP",
		width = 12, height = 12, scale = 0.65, depth = 8, offsetY = -2,
		providers = {
			condition = { class = "unit_condition", name = "raid_icon" },
			texture = { class = "unit_icon", name = "raid_icon" },
		},
	},
}

local playerWidgets = {
	"base", "healthBar", "powerBar", "incomingBar", "nameText", "healthText", "powerText", "classText", "pvpText",
	"combatIcon", "restIcon", "pvpIcon", "raidIcon", "roleIcon", "leaderIcon", "looterIcon",
}

local normalWidgets = {
	"base", "healthBar", "powerBar", "incomingBar", "nameText", "healthText", "powerText", "classText", "comboText",
	"pvpIcon", "raidIcon", "roleIcon", "leaderIcon", "looterIcon", 
}

local smallWidgets = {
	"base", "healthBar", "powerBar", "incomingBar", "nameText", "classText", "pvpIcon", "raidIcon",
}

local raidWidgets = {
	"base", "healthBar", "powerBar", "incomingBar", "nameText", "statusText", "readyIcon", "pvpIcon", "raidIcon", "arrowIcon",
	"icon1", "icon2", "icon3", "icon4", "icon5", "icon6", "icon7", "text1", "text2", "text3", "text4",
}
local raidSmallWidgets = {
	"base", "healthBar", "powerBar", "incomingBar", "nameText", "statusText", "raidIcon",
}

local raidTargetWidgets = {
	"base", "healthBar", "nameText", "raidIcon",
}

local panelWidgets = { "panel", "left_stripe", "right_stripe" }

local layout = {
	defaults = "standard", templates = "standard",
	anchors = {
		left_anchor = { x = 0, y = 0, anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT" },
		bottom_anchor = { x = 0, y = 0, anchorPoint = "BOTTOM", relativePoint = "BOTTOM" },
		right_anchor = { x = 0, y = 0, anchorPoint = "BOTTOMRIGHT", relativePoint = "BOTTOMRIGHT" },
		solo_anchor = { x = 0, y = 92, anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOM" },
		party_anchor = { x = 0, y = 500, anchorPoint = "BOTTOM", relativePoint = "BOTTOM" },
		raid_anchor = { x = 0, y = 240, anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT" },
		heal_anchor = { x = 160, y = 240, anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOM" },
		broker_anchor = { x = 800, y = 600, anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT" },
	},
	panels = {	
		bottom_bar = { anchor = "bottom_anchor", templates = "misc", widgets = "panel", width = 324, height = 30, offsetX = 0, offsetY = 6, depth = 0, },
		-- left_stripe = { anchor = "left_anchor", templates = "misc", widgets = "panel", width = 470, height = 4, offsetX = 8, offsetY = 10, depth = 0, },
		-- right_stripe = { anchor = "right_anchor", templates = "misc", widgets = "panel", width = 470, height = 4, offsetX = -8, offsetY = 10, depth = 0, },
	},
	units = {	
		player = { anchor = "solo_anchor", widgets = "player", width = 150, height = 36, offsetX = -160, offsetY = 28, depth = 1, clique = true },
		pet = { anchor = "solo_anchor", widgets = "small", width = 120, height = 36, offsetX = -287, offsetY = -36, depth = 1, },
		pettarget = { anchor = "solo_anchor", templates = "raid", widgets = "raid_small", width = 75, height = 24, offsetX = -242, offsetY = -64, depth = 1, },
		target = { anchor = "solo_anchor", widgets = "normal", width = 150, height = 36, offsetX = 10, offsetY = 28, depth = 1, },
		targettarget = { anchor = "solo_anchor", templates = "raid", widgets = "raid_small", width = 75, height = 24, offsetX = 10, offsetY = 0, depth = 1, },
		focus = { anchor = "solo_anchor", widgets = "small", width = 120, height = 36, offsetX = 167, offsetY = -36, depth = 1, },
		focustarget = { anchor = "solo_anchor", templates = "raid", widgets = "raid_small", width = 75, height = 24, offsetX = 167, offsetY = -64, depth = 1, },
	},
	party = {
		anchor = "raid_anchor", heal_anchor = "heal_anchor", templates = "raid", depth = 1, offsetX = 62, offsetY = 0, player = true,
		party = { widgets = "raid", width = 60, height = 24, offsetX = 0, offsetY = 14, clique = true },
		target = { templates = "raid_target", widgets = "raid_target", width = 60, height = 13, offsetX = 0, offsetY = 1 },
	},
	raid = {
		anchor = "raid_anchor", heal_anchor = "heal_anchor", templates = "raid", depth = 1,
		offsetX = 62, offsetY = 0, groupOffsetX = 0, groupOffsetY = 26, rowPoint = "LEFT", columnPoint = "TOP",
		player = { widgets = "raid", width = 60, height = 24, offsetX = 0, offsetY = 0, clique = true },
	},
}

-- Layout for data broker display. Requires knowing the names exported by the brokers plus static sizing of the text field.
-- Work list: allow multiple anchors, support secure brokers like Bugsack, support variable length text, support label field
local broker_layout = {
	defaults = "standard", templates = "broker", widgets = "broker",
	depth = 0, dirX = -1, dirY = 0, spaceX = 4, spaceY = 0, iconSize = 16,
	anchor = { x = -16, y = 1.5, anchorPoint = "BOTTOMLEFT", relativePoint = "BOTTOMRIGHT" },
	button = { class = "Widget_Button", relativeWidth = 1, relativeHeight = 1, height = 6, depth = 0, },
	text = {
		class = "Widget_Text", relativeWidth = 0, relativeHeight = 1, depth = 6,
		attributes = { fsize = 10, justifyH = "LEFT", justifyV = "MIDDLE", color = defaultYellow  },
	},
	brokers = {
		["Clock"] = { order = 1, icon = false, text = true, textWidth = 24 },
		["Switcher"] = { order = 2, icon = true, text = false },
		["Raven"] = { order = 3, icon = true, text = false },
		["Bartender4"] = { order = 4, icon = true, text = false },
		["Skada"] = { order = 5, icon = true, text = false },
		["AtlasLoot"] = { order = 6, icon = true, text = false },
		["Altoholic"] = { order = 7, icon = true, text = false },
		["Masque"] = { order = 8, icon = true, text = false },
		["Guild"] = { order = 12, icon = true, text = false },
		["BugSack"] = { order = 14, icon = true, text = false },
		["Mail"] = { order = 15, icon = true, text = false },
		["Memory"] = { order = 18, icon = false, text = true, textWidth = 40 },
		["Latency"] = { order = 19, icon = false, text = true, textWidth = 32 },
		["FPS"] = { order = 20, icon = false, text = true, textWidth = 32 },
	},
}

-- Generic layout for an icon group
local icon_group_layout = {
	defaults = "standard",
	orientation = "horizontal", direction = "left", center = false, limit = 40, spacing = 3,
	wrap = 10, wrapDirection = "down", scale = 1, alpha = 1,
	widgets = {
		icon = { -- need to include cooldown and border and start/end animations
			class = "Widget_Icon", width = 16, height = 16, depth = 8, offsetX = 0, offsetY = 0,
		},
		timer = {
			class = "Widget_Text", width = 64, height = 16, depth = 6, offsetX = 0, offsetY = 0, shadow = true, timeFormat = nil,
			attributes = { fsize = 12, justifyH = "CENTER", justifyV = "MIDDLE", color = defaultWhite  },
		},
		count = {
			class = "Widget_Text", width = 64, height = 16, depth = 7, offsetX = 0, offsetY = 0, shadow = true, outline = true, timeFormat = nil,
			attributes = { fsize = 12, justifyH = "CENTER", justifyV = "MIDDLE", color = defaultYellow  },
		},
	},
}

-- Active icon groups
local icon_groups = {
	["Buffs"] = {
		anchor = { x = -200, y = -32, anchorPoint = "TOPRIGHT", relativePoint = "TOPRIGHT" },
		sources = { -- data sources for spell list
			[1] = {data = "buffs", unit = "player", caster = nil, },
			[2] = {data = "cooldowns", },
		},
		status = nil, -- status tests for show/hide of icon group (e.g., in combat, in party, player class)
		filters = nil, -- tests applied to individual spells (e.g., black list, white list, grouped spells, type checks, time constraints)
		sorters = { [1] = "TimeUp", [2] = "AtoZ" }, -- ordered list of sort algorithms to apply
		colors = nil, -- colors that override defaults for icon border and timer/count text
		specialEffects = nil, -- animations, color changes, start and expiration sounds
		tooltips = nil, -- tooltip anchor, interactivity in and out of combat, annotation options (e.g., caster for auras)
	}
}

-- Add default and layout configuration data to the database
function Adapter_Configuration.OnEnable()
	DB = Puffin.Component("Data") -- link to data management functions
	assert(DB, EF("Adapter_Configuration", "Data component not found"))
	
	local class = Puffin.myClass -- check if using heal anchors
	local tree = nil
	local index = GetSpecialization()
	if (class == "DRUID" and index == 4) or (class == "PRIEST" and index == 2) then
		Puffin.myRole = "healer"
		layout.party.anchor = layout.party.heal_anchor; layout.raid.anchor = layout.raid.heal_anchor
	end
	
	DB.AllocateDatabase("defaults", false)
	DB.SetAttribute("defaults", "colors", "mana", mana_color)
	DB.SetAttribute("defaults", "colors", "level", defaultYellow)
	DB.SetAttribute("defaults", "colors", "race", defaultLight)
	DB.SetAttribute("defaults", "colors", "mouseover", mouseover_color)
	DB.SetAttribute("defaults", "number", "raid_groups_max", 6)
	DB.SetAttribute("defaults", "number", "out_of_range_alpha", 0.5)
	DB.SetAttribute("defaults", "Masque", "enable", true)
	
	DB.AllocateDatabase("widget_defaults", false)
	DB.SetRecord("widget_defaults", "standard", defaults)

	DB.AllocateDatabase("widget_templates", false)
	DB.SetRecord("widget_templates", "standard", standard_templates) 
	DB.SetRecord("widget_templates", "raid", raid_templates)
	DB.SetRecord("widget_templates", "raid_target", raid_target_templates)
	DB.SetRecord("widget_templates", "misc", misc_templates) 

	DB.AllocateDatabase("widgets", false)
	DB.SetRecord("widgets", "player", playerWidgets)
	DB.SetRecord("widgets", "normal", normalWidgets)
	DB.SetRecord("widgets", "small", smallWidgets)
	DB.SetRecord("widgets", "raid", raidWidgets)
	DB.SetRecord("widgets", "raid_small", raidSmallWidgets)
	DB.SetRecord("widgets", "raid_target", raidTargetWidgets)
	DB.SetRecord("widgets", "panel", panelWidgets)

	DB.AllocateDatabase("layouts", false)
	DB.SetRecord("layouts", "standard", layout)
	DB.SetRecord("layouts", "brokers", broker_layout)
	
	DB.AllocateDatabase("icon_groups", false)
	DB.SetRecord("icon_groups", "layout", icon_group_layout)
	DB.SetRecord("icon_groups", "groups", icon_groups)
end

Puffin.RegisterComponent("Adapter_Configuration", Adapter_Configuration, "ADAPTER", 0)
