-- Puffin: A Tojaso FowlPlay Project
-- Model component for tracking player-specific information

local Model_Player = {}
local DB = nil
local stance_changed = true
local talent_changed = true
local specialization_changed = true

local function StanceEvent() stance_changed = true end
local function TalentEvent() talent_changed = true end

function Model_Player.OnEnable()
	DB = Puffin.Component("Data") -- link to data management functions
	assert(DB, "Puffin.Model_Player: invalid reference to data management component")
	DB.AllocateDatabase("player", false)
	
	Puffin.RegisterCallback("UPDATE_SHAPESHIFT_FORM", StanceEvent)
	Puffin.RegisterCallback("PLAYER_TALENT_UPDATE", TalentEvent)
	Puffin.RegisterCallback("PLAYER_SPECIALIZATION_CHANGED", TalentEvent)	
end

function Model_Player.OnUpdate()
	if stance_changed then
		stance_changed = false
		local stance = GetShapeshiftForm()
		if stance and stance > 0 then
			local _, name = GetShapeshiftFormInfo(stance)
			if name then
				local icon = GetSpellTexture(name)
				local link = GetSpellLink(name)
				DB.SetAttribute("player", "stance", "name", name)
				DB.SetAttribute("player", "stance", "icon", icon)
				DB.SetAttribute("player", "stance", "link", link)
			end
		end
	end
	
	if talent_changed then
		talent_changed = false
		local tabs = GetNumSpecializations(false, false)
		if tabs >= 0 then
			local currentSpec = GetSpecialization()
			local talentSpec = currentSpec and select(2, GetSpecializationInfo(currentSpec)) or "None"
			DB.SetAttribute("player", "specialization", "name", talentSpec)		
		
			local ts = GetNumTalents(currentSpec)
			for i = 1, ts do
				local name, _, _, _, selected = GetTalentInfo(i) -- player's active talents
				if name then
					DB.SetAttribute("player", "talent", name, selected)
				end
			end
		end
	end
end

Puffin.RegisterComponent("Model_Player", Model_Player, "MODEL", 1)
